@@ -45,12 +45,14 @@ sub get_runlevels
      "mandrake-9.0"     => "redhat-6.2",
      "conectiva-9"      => "redhat-6.2",
      "debian-3.0"       => "redhat-6.2",
+     "ubuntu-7.04"      => "redhat-6.2",
      "suse-9.0"         => "redhat-6.2",
      "pld-1.0"          => "redhat-6.2",
      "vine-3.0"         => "redhat-6.2",
-     "slackware-9.1.0"  => "slackware-9.1.0",
+     "slackware-9.1.0"  => "freebsd-5",
      "gentoo"           => "gentoo",
-     "freebsd-5"        => "slackware-9.1.0",
+     "freebsd-5"        => "freebsd-5",
+     "solaris-2.11"     => "freebsd-5",
     );
 
   %runlevels=
@@ -80,7 +82,7 @@ sub get_default_runlevel
 {
   my $type = &get_init_type ();
 
-  return "default" if ($type eq "gentoo" || $type eq "rcng" || $type eq "bsd");
+  return "default" if ($type eq "gentoo" || $type eq "rcng" || $type eq "bsd" || $type eq "smf");
   return &get_sysv_default_runlevel ();
 }
 
@@ -99,10 +101,12 @@ sub get_sysv_paths
      "yoper-2.2"      => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "conectiva-9"    => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "debian-3.0"     => ["$gst_prefix/etc",      "$gst_prefix/etc/init.d",      "../init.d"],
+     "ubuntu-7.04"    => ["$gst_prefix/etc",      "$gst_prefix/etc/init.d",      "../init.d"],
      "suse-9.0"       => ["$gst_prefix/etc/init.d", "$gst_prefix/etc/init.d",    "../"],
      "pld-1.0"        => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "vine-3.0"       => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
      "ark"            => ["$gst_prefix/etc/rc.d", "$gst_prefix/etc/rc.d/init.d", "../init.d"],
+     "solaris-2.11"   => ["$gst_prefix/etc",      "$gst_prefix/etc/init.d",      "../init.d"],
      );
   my $res;
 
@@ -174,7 +178,6 @@ sub get_sysv_services
 	my (@arr);
 
 	($rcd_path, $initd_path) = &get_sysv_paths ();
-
   return undef unless ($rcd_path && $initd_path);
 
 	foreach $service (<$initd_path/*>)
@@ -223,6 +226,7 @@ sub run_sysv_initd_script
   &Utils::Report::enter ();
   
   ($rcd_path, $initd_path) = &get_sysv_paths ();
+  return -1 unless ($rcd_path && $initd_path);
 
   if (-f "$initd_path/$service")
   {
@@ -246,6 +250,7 @@ sub set_sysv_service
   my ($runlevel, $action, %configured_runlevels);
 
   ($rcd_path, $initd_path, $relative_path) = &get_sysv_paths ();
+  return unless ($rcd_path && $initd_path && $relative_path);
 
   $script = $$service[0];
   $runlevels = $$service[1];
@@ -456,6 +461,7 @@ sub set_filerc_services
   my ($services) = @_;
   my ($buff, $lineno, $line, $file);
   my ($rcd_path, $initd_path, $relative_path) = &get_sysv_paths ();
+  return unless ($rcd_path && $initd_path && $relative_path);
 
   $file = "/etc/runlevel.conf";
 
@@ -500,7 +506,8 @@ sub set_filerc_services
 sub get_bsd_scripts_list
 {
   my ($files) = [ "rc.M", "rc.inet2", "rc.4" ];
-  my ($file, $i, @scripts);
+  my ($file, $i, %scripts);
+  my ($service, $name);
 
   foreach $i (@$files)
   {
@@ -509,7 +516,7 @@ sub get_bsd_scripts_list
 
     if (!$fd) {
       &Utils::Report::do_report ("rc_file_read_failed", $file);
-      return undef;
+      next;
     }
 
     while (<$fd>)
@@ -519,54 +526,50 @@ sub get_bsd_scripts_list
       if ($line =~ /^if[ \t]+\[[ \t]+\-x[ \t]([0-9a-zA-Z\/\.\-_]+) .*\]/)
       {
         $service = $1;
+        $name = $service;
+        $name =~ s/^.*\///;
+        $name =~ s/^rc\.//;
 
-        push @scripts, $service;
+        $scripts{$name} = $service;
       }
     }
 
-    Utils::File::close_file ($fd);
+    &Utils::File::close_file ($fd);
   }
 
-  return \@scripts;
+  return \%scripts;
+}
+
+sub get_bsd_service_status
+{
+  my ($service) = @_;
+  return (-x $service) ? $SERVICE_START : $SERVICE_STOP;
 }
 
 sub get_bsd_service_info
 {
-  my ($service) = @_;
-  my ($script, @runlevels);
+  my ($service, $name) = @_;
+  my (@runlevels, $status);
 
-  $script = $service;
-  $script =~ s/^.*\///;
-  $script =~ s/^rc\.//;
+  return if (! Utils::File::exists ($service));
 
-  return undef if (! Utils::File::exists ($service));
+  $status = &get_bsd_service_status ($service);
+  push @runlevels, [ "default", $status, 0 ];
 
-  # hardcode the fourth runlevel, it's the graphical one
-  if ( -x $service)
-  {
-    push @runlevels, [ "4", $SERVICE_START, 0 ];
-  }
-  else
-  {
-    push @runlevels, [ "4", $SERVICE_STOP, 0 ];
-  }
-
-  # FIXME: $service contains full path, would
-  # be nice to just show the $script name
-  return ($service, $runlevels);
+  return ($name, \@runlevels);
 }
 
 sub get_bsd_services
 {
-  my (@arr, @scripts, $script);
+  my (@arr, %scripts, $name);
 
   $scripts = &get_bsd_scripts_list ();
 
-  foreach $script (@scripts)
+  foreach $name (keys %$scripts)
   {
     my (@info);
 
-    @info = &get_bsd_service_info ($script);
+    @info = &get_bsd_service_info ($$scripts{$name}, $name);
     push @arr, \@info if (scalar (@info));
   }
 
@@ -587,7 +590,7 @@ sub run_bsd_script
     &Utils::File::run ("chmod ugo+x $service");
   }
   
-  &Utils::File::run ("$service $arg");
+  &Utils::File::run ("$service $arg", 1);
 
   # return it to it's normal state
   if ($chmod)
@@ -597,41 +600,28 @@ sub run_bsd_script
 }
 
 # This function stores the configuration in a bsd init
-sub get_bsd_service_path
-{
-  my ($script) = @_;
-  my (@scripts, $elem, $i);
-
-  $scripts = &get_bsd_scripts_list ();
-
-  foreach $i (@scripts)
-  {
-    $elem = $i;
-    $elem =~ s/^.*\///;
-    $elem =~ s/^rc\.//;
-
-    return $i if ($elem eq $script);
-  }
-
-  return undef;
-}
-
 sub set_bsd_services
 {
   my ($services) = @_;
-  my ($script, $runlevels, @scripts);
+  my ($script, $runlevels, %scripts);
+  my ($status);
+
+  $scripts = &get_bsd_scripts_list ();
 
   foreach $service (@$services)
   {
-    $script = &get_bsd_service_path ($$service[0]);
+    $script = $$scripts{$$service[0]};
     $runlevels = $$service[1];
     $runlevel  = $$runlevels[0];
 
     next if ($script eq undef);
 
-    $action = $$runlevel[1];
+    $status = $$runlevel[1];
 
-    if ($action == $SERVICE_START)
+    next if ($status == &get_bsd_service_status ($script));
+
+    if ($status ne undef &&
+        $status == $SERVICE_START)
     {
       &Utils::File::run ("chmod ugo+x $script");
       &run_bsd_script ($script, "start");
@@ -864,7 +854,7 @@ sub get_rcng_status_by_service
   my ($fd, $line, $active);
 
   # This is the only difference between rcNG and archlinux
-  if ($gst_dist eq "archlinux")
+  if ($Utils::Backend::tool{"platform"} eq "archlinux")
   {
       return &Utils::File::exists ("/var/run/daemons/$service");
   }
@@ -883,7 +873,7 @@ sub get_rcng_status_by_service
       }
     }
 
-    Utils::File::close_file ($fd);
+    &Utils::File::close_file ($fd);
     return $active;
   }
 }
@@ -1019,7 +1009,7 @@ sub set_rcng_services
   my ($action, $runlevels, $script, $func);
 
   # archlinux stores services differently
-  if ($gst_dist eq "archlinux")
+  if ($Utils::Backend::tool{"platform"} eq "archlinux")
   {
     $func = \&set_archlinux_service_status;
   }
@@ -1044,7 +1034,7 @@ sub get_suse_service_info ($service)
 {
   my ($service) = @_;
   my (@runlevels, $link, $runlevel);
-                                                                                                                                                             
+
   foreach $link (<$rcd_path/rc[0-9S].d/S[0-9][0-9]$service>)
   {
     $link =~ s/$rcd_path\///;
@@ -1067,6 +1057,7 @@ sub get_suse_services
   my ($service, @arr);
 
   ($rcd_path, $initd_path) = &get_sysv_paths ();
+  return undef unless ($rcd_path && $initd_path);
 
   foreach $service (<$gst_prefix/etc/init.d/*>)
   {
@@ -1126,6 +1117,158 @@ sub set_suse_services
   }
 }
 
+# functions to get/set services info in smf
+sub smf_service_exists
+{
+  my($service) = @_;
+  my($services) = &get_smf_services_list ();
+
+  foreach $i (@$services)
+  {
+    return 1 if ($i =~ /$service/);
+  }
+
+  return 0;
+}
+
+sub run_smf_svcadm
+{
+  my ($service, $arg) = @_;
+  my ($option);
+
+  my %op =
+    ("stop" => "disable",
+     "start" => "enable"
+    );
+
+  if (&smf_service_exists ($service))
+  {
+    if (!&Utils::File::run ("svcadm $op{$arg} $service"))
+    {
+      &Utils::Report::do_report ("service_sysv_op_success", $service, $arg);
+      &Utils::Report::leave ();
+      return 0;
+    }
+  }
+
+  &Utils::Report::do_report ("service_sysv_op_failed", $service, $arg);
+  &Utils::Report::leave ();
+  return -1;
+}
+
+sub get_smf_runlevel_status_by_service
+{
+  my ($service, $status) = @_;
+  my (@arr);
+
+  if ($status)
+  {
+    push @arr, [ "default", $SERVICE_START, 0 ];
+  }
+  else
+  {
+    push @arr, [ "default", $SERVICE_STOP, 0 ];
+  }
+
+  return \@arr;
+}
+
+sub get_smf_service_info
+{
+  my ($service) = @_;
+  my ($fd, @runlevels);
+  my $status = 0;
+
+  $fd = &Utils::File::run_pipe_read ("svcs -l $service");
+
+  while (<$fd>)
+  {
+    $status = 1 if (/^state.*online/);
+  }
+
+  &Utils::File::close_file ($fd);
+
+  $runlevels = &get_smf_runlevel_status_by_service ($service, $status);
+  $service =~ m/.*\/(.*)$/;
+
+  return ($service, $runlevels);
+}
+
+sub get_smf_services_list
+{
+  my ($fd, @list);
+
+  $fd = &Utils::File::run_pipe_read ("svcs -H -a");
+
+  while (<$fd>)
+  {
+    next if (/svc:\/milestone/);
+
+    if (/^.*\s*.*\s*svc:\/(.*):.*/)
+    {
+      push @list, $1;
+    }
+  }
+
+  &Utils::File::close_file ($fd);
+  return \@list;
+}
+
+sub get_smf_services
+{
+  my ($service, @arr);
+  my ($service_list) = &get_smf_services_list ();
+
+  foreach $service (@$service_list)
+  {
+    my (@info);
+
+    @info = &get_smf_service_info ($service);
+    push @arr, \@info if scalar (@info);
+  }
+
+  return \@arr;
+}
+
+sub set_smf_service_status
+{
+  my ($service, $rl, $active) = @_;
+  my ($info);
+
+  $info = &get_smf_service_info ($service);
+
+  #return if service has not changed
+  return if ($active == @{@$info[0]}[1]);
+
+  if ($active == $SERVICE_START)
+  {
+    &Utils::File::run ("svcadm enable -s $service");
+  }
+  else
+  {
+    &Utils::File::run ("svcadm disable -s $service");
+  }
+}
+
+sub set_smf_services
+{
+  my ($services) = @_;
+  my ($action, $rl, $script, $arr);
+
+  foreach $service (@$services)
+  {
+    $script = $$service[0];
+    $arr = $$service[1];
+
+    foreach $i (@$arr)
+    {
+      $action = $$i[1];
+      $rl = $$i[0];
+      &set_smf_service_status ($script, $rl, $action);
+    }
+  }
+}
+
 # generic functions to get the available services
 sub get_init_type
 {
@@ -1153,6 +1296,10 @@ sub get_init_type
   {
     return "suse";
   }
+  elsif ($gst_dist =~ /solaris/)
+  {
+    return "smf";
+  }
   else
   {
     return "sysv";
@@ -1171,6 +1318,7 @@ sub run_script
      "gentoo"  => \&run_gentoo_script,
      "rcng"    => \&run_rcng_script,
      "suse"    => \&run_sysv_initd_script,
+     "smf"     => \&run_smf_svcadm,
     );
 
   $type = &get_init_type ();
@@ -1188,6 +1336,7 @@ sub get
   return &get_gentoo_services () if ($type eq "gentoo");
   return &get_rcng_services ()   if ($type eq "rcng");
   return &get_suse_services ()   if ($type eq "suse");
+  return &get_smf_services ()    if ($type eq "smf");
 
   return undef;
 }
@@ -1204,6 +1353,7 @@ sub set
   &set_gentoo_services ($services) if ($type eq "gentoo");
   &set_rcng_services   ($services) if ($type eq "rcng");
   &set_suse_services   ($services) if ($type eq "suse");
+  &set_smf_services    ($services) if ($type eq "smf");
 }
 
 1;
